#region File Description
//-----------------------------------------------------------------------------
// BackgroundScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace TetrisDreams
{
    class FallingBlock
        {
        public int type;
        public Vector3 pos;
        public Vector3 rot;
        public FallingBlock(int Type, Vector3 Pos, Vector3 Rot)
            {
            type = Type;
            pos = Pos;
            rot = Rot;
            }

        }

    /// <summary>
    /// The background screen sits behind all the other menu screens.
    /// It draws a background image that remains fixed in place regardless
    /// of whatever transitions the screens on top of it may be doing.
    /// </summary>
    class BackgroundScreen : GameScreen
    {
        #region Fields

        ContentManager content;
        private Model block_t;
        private Model block_o;
        private Model block_i;
        private Model block_s;
        private Model block_z;
        private Model block_l;
        private Model block_j;

        private Matrix world = Matrix.CreateTranslation(new Vector3(0, 0, 0));
        private Matrix view = Matrix.CreateLookAt(new Vector3(0, 0, 10), new Vector3(0, 0, 0), Vector3.UnitY);
        private Matrix projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45), 1920f / 1080f, 0.1f, 100f);
        private int angle;

        private FallingBlock[] blockRow = new FallingBlock[10];
        private FallingBlock[] blockRow2 = new FallingBlock[10];
        private FallingBlock[] blockRow3 = new FallingBlock[10];
        private FallingBlock[] blockRow4 = new FallingBlock[10];
        private static Model[] blockModels = new Model[7];
        private float scale = 0.2f;


        #endregion

        #region Initialization


        /// <summary>
        /// Constructor.
        /// </summary>
        public BackgroundScreen()
            {
            TransitionOnTime = TimeSpan.FromSeconds(0.5);
            TransitionOffTime = TimeSpan.FromSeconds(0.5);

            for(int i=0; i<blockRow.Length; i++)
                blockRow[i] = new FallingBlock(i>6 ? i-7 : i, new Vector3(i*5-20, i*5-20, 0), new Vector3(0, 1, 0));

            for (int i=0; i < blockRow2.Length; i++)
                blockRow2[i] = new FallingBlock(i>6 ? i-7 : i, new Vector3(i*5-20, i*5-20, 0), new Vector3());

            for (int i = 0; i < blockRow3.Length; i++)
                blockRow3[i] = new FallingBlock(i>6 ? i-7 : i, new Vector3(i*5-20, i*5-20, 0), new Vector3());

            for (int i = 0; i < blockRow4.Length; i++)
                blockRow4[i] = new FallingBlock(i>6 ? i-7 : i, new Vector3(i*5-20, i*5-20, 0), new Vector3());
            }


        /// <summary>
        /// Loads graphics content for this screen. The background texture is quite
        /// big, so we use our own local ContentManager to load it. This allows us
        /// to unload before going from the menus into the game itself, wheras if we
        /// used the shared ContentManager provided by the Game class, the content
        /// would remain loaded forever.
        /// </summary>
        public override void LoadContent()
            {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");
       
            angle = 0;
            blockModels[0] = block_t = content.Load<Model>("block_t");
            blockModels[1] = block_o = content.Load<Model>("block_o");
            blockModels[2] = block_i = content.Load<Model>("block_i");
            blockModels[3] = block_s = content.Load<Model>("block_s");
            blockModels[4] = block_z = content.Load<Model>("block_z");
            blockModels[5] = block_l = content.Load<Model>("block_l");
            blockModels[6] = block_j = content.Load<Model>("block_j");
            }


        /// <summary>
        /// Unloads graphics content for this screen.
        /// </summary>
        public override void UnloadContent()
            {
            content.Unload();
            }


        #endregion

        #region Update and Draw


        /// <summary>
        /// Updates the background screen. Unlike most screens, this should not
        /// transition off even if it has been covered by another screen: it is
        /// supposed to be covered, after all! This overload forces the
        /// coveredByOtherScreen parameter to false in order to stop the base
        /// Update method wanting to transition off.
        /// </summary>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                                       bool coveredByOtherScreen)
            {
            base.Update(gameTime, otherScreenHasFocus, false);

            if (gameTime.TotalGameTime.Seconds % 4 == 0 || angle % 90 != 0)
                {
                angle += 1;
                }

            for (int i = 0; i < blockRow.Length; i++)
                {
                blockRow[i].pos -= new Vector3(0.1f, 0.1f, 0);
                if(blockRow[i].pos.Y < -25)
                    blockRow[i].pos.X = blockRow[i].pos.Y += 50;
                }

            for (int i = 0; i < blockRow2.Length; i++)
                {
                blockRow2[i].pos -= new Vector3(0.1f, 0.1f, 0);
                if(blockRow2[i].pos.Y < -25)
                    {
                    blockRow2[i].pos.X += 50;
                    blockRow2[i].pos.Y += 50;
                    }
                }

            for (int i = 0; i < blockRow3.Length; i++)
                {
                blockRow3[i].pos -= new Vector3(0.1f, 0.1f, 0);
                if (blockRow3[i].pos.Y < -30 || 
                    blockRow3[i].pos.X < -25)
                    {
                    blockRow3[i].pos.X += 50;
                    blockRow3[i].pos.Y += 50;
                    }
                }

            for (int i = 0; i < blockRow4.Length; i++)
                {
                blockRow4[i].pos -= new Vector3(0.1f, 0.1f, 0);
                if (blockRow4[i].pos.Y < -30 ||
                    blockRow4[i].pos.X < -25)
                    {
                    blockRow4[i].pos.X += 50;
                    blockRow4[i].pos.Y += 50;
                    }

                }

            }


        /// <summary>
        /// Draws the background screen.
        /// </summary>
        public override void Draw(GameTime gameTime)
            {
            SpriteBatch spriteBatch = ScreenManager.SpriteBatch;
            Viewport viewport = ScreenManager.GraphicsDevice.Viewport;
            Rectangle fullscreen = new Rectangle(0, 0, viewport.Width, viewport.Height);

            spriteBatch.Begin();

            GraphicsDevice graphics = ScreenManager.GraphicsDevice;

            graphics.Clear(Color.Black);
            
            spriteBatch.End();

            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, null, null, null, null, Resolution.getTransformationMatrix());

            // Fix to get correct 3D depth rendering
            //graphics.BlendState = BlendState.Opaque;
            graphics.DepthStencilState = DepthStencilState.Default;
            graphics.SamplerStates[0] = SamplerState.LinearWrap;

            for (int i =0; i < blockRow.Length; i++)
                {
                DrawModel(blockModels[ (int)blockRow[i].type ],
                          world * Matrix.CreateRotationZ(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(blockRow[i].pos) * Matrix.CreateScale(scale),
                          view, 
                          projection);
                }

            for (int i = 0; i < blockRow2.Length; i++)
                {

                DrawModel(blockModels[(int)blockRow2[i].type],
                          world * Matrix.CreateRotationZ(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(blockRow2[i].pos + new Vector3(15, 0, 0)) * Matrix.CreateScale(scale) * Matrix.CreateRotationZ(MathHelper.ToRadians(180)),
                          view,
                          projection);
                DrawModel(blockModels[(int)blockRow2[i].type],
                          world * Matrix.CreateRotationZ(MathHelper.ToRadians(angle)) *Matrix.CreateTranslation(blockRow2[i].pos + new Vector3(-15, 0, 0)) * Matrix.CreateScale(scale) * Matrix.CreateRotationZ(MathHelper.ToRadians(180)),
                          view,
                          projection);
                }

            for (int i = 0; i < blockRow3.Length; i++)
                {

                DrawModel(blockModels[(int)blockRow3[i].type],
                          world * Matrix.CreateRotationZ(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(blockRow3[i].pos + new Vector3(30, 0, 0)) * Matrix.CreateScale(scale),
                          view,
                          projection);
                DrawModel(blockModels[(int)blockRow3[i].type],
                          world * Matrix.CreateRotationZ(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(blockRow3[i].pos + new Vector3(-30, 0, 0)) * Matrix.CreateScale(scale),
                          view,
                          projection);
                }

            for (int i = 0; i < blockRow4.Length; i++)
                {

                DrawModel(blockModels[(int)blockRow4[i].type],
                          world * Matrix.CreateRotationZ(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(blockRow4[i].pos + new Vector3(45, 0, 0)) * Matrix.CreateScale(scale) * Matrix.CreateRotationZ(MathHelper.ToRadians(180)),
                          view,
                          projection);
                DrawModel(blockModels[(int)blockRow4[i].type],
                          world * Matrix.CreateRotationZ(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(blockRow4[i].pos + new Vector3(-45, 0, 0)) * Matrix.CreateScale(scale) * Matrix.CreateRotationZ(MathHelper.ToRadians(180)),
                          view,
                          projection);
                }

            spriteBatch.End();
            }


        #endregion

        private void DrawModel(Model model, Matrix world, Matrix view, Matrix projection)
            {
            foreach (ModelMesh mesh in model.Meshes)
                {
                foreach (BasicEffect effect in mesh.Effects)
                    {
                    //effect.EnableDefaultLighting();
                    /*
                    effect.LightingEnabled = true; // turn on the lighting subsystem.
                    effect.DirectionalLight0.DiffuseColor = new Vector3(0.5f, 0, 0); // a red light
                    effect.DirectionalLight0.Direction = new Vector3(1, 0, 0);  // coming along the x-axis
                    effect.DirectionalLight0.SpecularColor = new Vector3(0, 1, 0); // with green highlights
                    effect.DirectionalLight0.Enabled = false;
                    
                    effect.AmbientLightColor = new Vector3(0.4f, 0.4f, 1.0f);
                    effect.EmissiveColor = new Vector3(0, 0, 0.1f);
                    /*
                    effect.FogEnabled = true;
                    effect.FogColor = Color.SlateGray.ToVector3(); // For best results, ake this color whatever your background is.
                    effect.FogStart = 9f;
                    effect.FogEnd = 18f;*/

                    effect.Alpha = 0.3f;
                    
                    effect.World = world;
                    effect.View = view;
                    effect.Projection = projection;
                    }

                mesh.Draw();
                }
            }
        }
}
