#region File Description
//-----------------------------------------------------------------------------
// HighscoreScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System;
using System.IO;
using System.Diagnostics;
using System.Xml.Serialization;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace TetrisDreams
{
    /// <summary>
    /// The Highscore screen displayes the ten best scores in a classic
    /// highscore table.
    /// </summary>
    class HighscoreScreen : GameScreen
    {
        #region Fields

        ContentManager content;

        //bool loadDone = false;

        #endregion

        #region Initialization


        /// <summary>
        /// Constructor.
        /// </summary>
        public HighscoreScreen()
            {
            }


        /// <summary>
        /// Loads graphics content for this screen. The background texture is quite
        /// big, so we use our own local ContentManager to load it. This allows us
        /// to unload before going from the menus into the game itself, wheras if we
        /// used the shared ContentManager provided by the Game class, the content
        /// would remain loaded forever.
        /// </summary>
        public override void LoadContent()
            {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");
            }


        /// <summary>
        /// Unloads graphics content for this screen.
        /// </summary>
        public override void UnloadContent()
            {
            content.Unload();
            }


        #endregion

        #region Update and Draw


        /// <summary>
        /// Updates the background screen. Unlike most screens, this should not
        /// transition off even if it has been covered by another screen: it is
        /// supposed to be covered, after all! This overload forces the
        /// coveredByOtherScreen parameter to false in order to stop the base
        /// Update method wanting to transition off.
        /// </summary>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                                       bool coveredByOtherScreen)
            {
            base.Update(gameTime, otherScreenHasFocus, false);
            }

         public override void HandleInput(InputState input)
            {
            if (input == null)
                    throw new ArgumentNullException("input");

                // Look up inputs for the active player profile.
                PlayerIndex playerIndex = ControllingPlayer.Value;

                KeyboardState keyboardState = input.CurrentKeyboardStates[(int)playerIndex];
                GamePadState gamePadState = input.CurrentGamePadStates[(int)playerIndex];

                // The game pauses either if the user presses the pause button, or if
                // they unplug the active gamepad. This requires us to keep track of
                // whether a gamepad was ever plugged in, because we don't want to pause
                // on PC if they are playing with a keyboard and have no gamepad at all!
                bool gamePadDisconnected = !gamePadState.IsConnected &&
                                           input.GamePadWasConnected[(int)playerIndex];

                if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                    {
                    ScreenManager.RemoveScreen(this);
                    }
                if (input.IsPauseGame(ControllingPlayer) || gamePadDisconnected)
                    {              
                    ScreenManager.RemoveScreen(this);
                    }
                else
                {
                if (input.IsNewKeyPress(Keys.Escape, ControllingPlayer, out playerIndex))
                    {
                    ScreenManager.RemoveScreen(this);
                    }
                }
            }

        /// <summary>
        /// Draws the background screen.
        /// </summary>
        public override void Draw(GameTime gameTime)
            {
            SpriteBatch spriteBatch = ScreenManager.SpriteBatch;
            Viewport viewport = ScreenManager.GraphicsDevice.Viewport;
            Rectangle fullscreen = new Rectangle(0, 0, viewport.Width, viewport.Height);

            int height = viewport.Height;
            int width = viewport.Width;
            int middlescreenX = 1920 / 2;

            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, null, null, null, null, Resolution.getTransformationMatrix());

            SpriteFont font = ScreenManager.Font;
            Vector2 origin = new Vector2(0, font.LineSpacing / 2);
            Color color = TetrisBlock.tetrisColors[0];

            string highscore = "HIGHSCORE";

            spriteBatch.DrawString(font, highscore.ToString(), new Vector2(middlescreenX - font.MeasureString(highscore.ToString()).X/2-54, 150), color, 0, origin, 1.5f, SpriteEffects.None, 0);
            
            int posY = 0;
            foreach (HighscorePost scorepost in TetrisModel.Instance.tetrisSaveData.highscore)
                {
                posY++;
                String posString = posY.ToString();
               color = posY < 7 ? TetrisBlock.tetrisColors[posY] : TetrisBlock.tetrisColors[posY - 7];
               
                spriteBatch.DrawString(font, posString + ". ", new Vector2(middlescreenX - 130 - font.MeasureString(posString.ToString()).X, posY * 35 + 200), color, 0, origin, 1, SpriteEffects.None, 0);
                spriteBatch.DrawString(font, scorepost.name.ToString(), new Vector2(middlescreenX - 100, posY*35+200), color, 0, origin, 1, SpriteEffects.None, 0);
                spriteBatch.DrawString(font, scorepost.score.ToString(), new Vector2(middlescreenX + 200 - font.MeasureString(scorepost.score.ToString()).X, posY * 35 + 200), color, 0, origin, 1, SpriteEffects.None, 0);
                }

            int newScore = TetrisModel.Instance.newHighScore;
            if( newScore != -1 )
                {
                spriteBatch.DrawString(font, "new", new Vector2(middlescreenX - 280, (newScore+1) * 35 + 200), color, 0, origin, 1, SpriteEffects.None, 0);
                }

            spriteBatch.End();
            }


        #endregion
    }
}
