#region File Description
//-----------------------------------------------------------------------------
// OptionsMenuScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Input;
#endregion

namespace TetrisDreams
{
    /// <summary>
    /// The options screen is brought up over the top of the main menu
    /// screen, and gives the user a chance to configure the game
    /// in various hopefully useful ways.
    /// </summary>
    class OptionsMenuScreen : MenuScreen
    {
        #region Fields

        MenuEntry musicMenuEntry;
        MenuEntry musicVolymeMenuEntry;
        
        MenuEntry sfxMenuEntry;
        MenuEntry sfxVolymeMenuEntry;
        bool saveDone = true;

        bool music;
        int musicVolyme;
        bool sfx;
        int sfxVolyme;

        AudioEngine audioEngine;
        AudioCategory audioCategory;

        #endregion

        #region Initialization


        /// <summary>
        /// Constructor.
        /// </summary>
        public OptionsMenuScreen()
            : base("Options")
            {
            music = TetrisModel.Instance.tetrisSaveData.music;
            musicVolyme = TetrisModel.Instance.tetrisSaveData.musicvolyme;
            sfx = TetrisModel.Instance.tetrisSaveData.sfx;
            sfxVolyme = TetrisModel.Instance.tetrisSaveData.sfxvolyme;

            // Create our menu entries.
            musicMenuEntry = new MenuEntry(string.Empty);
            musicVolymeMenuEntry = new MenuEntry(string.Empty);
            sfxMenuEntry = new MenuEntry(string.Empty);
            sfxVolymeMenuEntry = new MenuEntry(string.Empty);
            SetMenuEntryText();

            MenuEntry back = new MenuEntry("Back");

            // Hook up menu event handlers.
            musicMenuEntry.Selected += MusicMenuEntrySelected;
            musicVolymeMenuEntry.Selected += MusicVolymeMenuEntrySelected;
            sfxMenuEntry.Selected += SfxMenuEntrySelected;
            sfxVolymeMenuEntry.Selected += SfxVolymeMenuEntrySelected;
            back.Selected += OnBack;

            // Add entries to the menu.
            MenuEntries.Add(musicMenuEntry);
            MenuEntries.Add(musicVolymeMenuEntry);
            MenuEntries.Add(sfxMenuEntry);
            MenuEntries.Add(sfxVolymeMenuEntry);
            MenuEntries.Add(back);

            audioEngine = new AudioEngine("Content\\sounds.xgs");
            audioCategory = audioEngine.GetCategory("Sfx");
            }


        /// <summary>
        /// Fills in the latest values for the options screen menu text.
        /// </summary>
        void SetMenuEntryText()
            {
            musicMenuEntry.Text = "Music: " + (music ? "on" : "off");
            musicVolymeMenuEntry.Text = "Music Volyme: " + musicVolyme;
            sfxMenuEntry.Text = "Sfx: " + (sfx ? "on" : "off");
            sfxVolymeMenuEntry.Text = "Sfx Volyme: " + sfxVolyme;
            }


        #endregion

        #region Handle Input


        /// <summary>
        /// Event handler for when the Ungulate menu entry is selected.
        /// </summary>
        void MusicMenuEntrySelected(object sender, PlayerIndexEventArgs e)
            {
            music = !music;
            TetrisModel.Instance.tetrisSaveData.music = music;
            if(music)
                {
                if(MediaPlayer.Queue.Count >= 1 )
                    MediaPlayer.Resume();
                else
                    TetrisModel.Instance.StartMusic(); 
                }
            else
                MediaPlayer.Pause();

            saveDone = false; // Todo only save at exit
            SetMenuEntryText();
            }


        /// <summary>
        /// Event handler for when the Language menu entry is selected.
        /// </summary>
        void MusicVolymeMenuEntrySelected(object sender, PlayerIndexEventArgs e)
            {
            musicVolyme--;
            if(musicVolyme<0)
                musicVolyme = 100;
            TetrisModel.Instance.tetrisSaveData.musicvolyme = musicVolyme;
            MediaPlayer.Volume = (float)musicVolyme / 100;
            saveDone = false; // Todo only save at exit
            SetMenuEntryText();
            }



        /// <summary>
        /// Event handler for when the Ungulate menu entry is selected.
        /// </summary>
        void SfxMenuEntrySelected(object sender, PlayerIndexEventArgs e)
            {
            sfx = !sfx;
            TetrisModel.Instance.tetrisSaveData.sfx = sfx;
            GraphicsDeviceManager GraphicsMan = GameStateManagementGame.graphics;
            saveDone = false; // Todo only save at exit
            SetMenuEntryText();
            }


        /// <summary>
        /// Event handler for when the Language menu entry is selected.
        /// </summary>
        void SfxVolymeMenuEntrySelected(object sender, PlayerIndexEventArgs e)
            {
            sfxVolyme--;
            if (sfxVolyme < 0)
                sfxVolyme = 100;
            TetrisModel.Instance.tetrisSaveData.sfxvolyme = sfxVolyme;

            audioCategory.SetVolume((float)sfxVolyme / 100);
            
            saveDone = false; // Todo only save at exit
            SetMenuEntryText();    
            }

        void OnBack(object sender, PlayerIndexEventArgs e)
            {
            OnCancel(sender, e);
            }

        #endregion

        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                                       bool coveredByOtherScreen)
            {
            base.Update(gameTime, otherScreenHasFocus, false);



            audioEngine.Update();

            if (!saveDone)
                {
                // Todo fix this to make sure we wont get a half done save 
                // if we exit the screen right after changing some option
                saveDone = TetrisModel.Instance.DoGameSave();
                }
            }
    }
}
