#region File Description
//-----------------------------------------------------------------------------
// TetrisDreams.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Media;
#endregion

namespace TetrisDreams
{
    /// <summary>
    /// Sample showing how to manage different game states, with transitions
    /// between menu screens, a loading screen, the game itself, and a pause
    /// menu. This main game class is extremely simple: all the interesting
    /// stuff happens in the ScreenManager component.
    /// </summary>
    public class GameStateManagementGame : Microsoft.Xna.Framework.Game
        {
        #region Fields

        public static GraphicsDeviceManager graphics;
        ScreenManager screenManager;
        public static Song music;
        bool loadDone = false;
        
        // By preloading any assets used by UI rendering, we avoid framerate glitches
        // when they suddenly need to be loaded in the middle of a menu transition.
        static readonly string[] preloadAssets =
            {
            "gradient",
            "tetris",
            };


        #endregion

        #region Initialization


        /// <summary>
        /// The main game constructor.
        /// </summary>
        public GameStateManagementGame()
            {
            Content.RootDirectory = "Content";

            // 1080p
            graphics = new GraphicsDeviceManager(this);
            //graphics.PreferredBackBufferWidth = 1920;
            //graphics.PreferredBackBufferHeight = 1080;

            // 720p
            //graphics.PreferredBackBufferWidth = 1280;
            //graphics.PreferredBackBufferHeight = 720;

            // 480p
            //graphics.PreferredBackBufferWidth = 640;
            //graphics.PreferredBackBufferHeight = 480;

            // Fullscreen?
            //graphics.IsFullScreen = true;
            

            //Fix to get rid multiple Draw() calls in a row
            //http://forums.create.msdn.com/forums/t/9934.aspx
            IsFixedTimeStep = false;
            graphics.SynchronizeWithVerticalRetrace = true;

            // Create the screen manager component.
            screenManager = new ScreenManager(this);
            screenManager.TraceEnabled = true;

            Components.Add(screenManager);

            #if XBOX
            Components.Add(new GamerServicesComponent(this));
            #endif

            
            music = Content.Load<Song>("tetris");
            }


        /// <summary>
        /// Loads graphics content.
        /// </summary>
        protected override void LoadContent()
            {
            int width = graphics.GraphicsDevice.DisplayMode.Width;
            int height = graphics.GraphicsDevice.DisplayMode.Height;
            
            Resolution.Init(ref graphics);
            Resolution.SetVirtualResolution(1920, 1080);
            Resolution.SetResolution(width, height, true);

            foreach (string asset in preloadAssets)
                {
                Content.Load<object>(asset);
                }
            }


        #endregion

        protected override void Update(GameTime gameTime)
            {
            if (!loadDone)
                {
                loadDone = TetrisModel.Instance.DoGameLoad();
                if(loadDone)
                    {  
                    // Activate the first screens.
                    screenManager.AddScreen(new BackgroundScreen(), null);
                    screenManager.AddScreen(new MainMenuScreen(), null);

                    // Start looping music
                    TetrisModel.Instance.StartMusic();
                    }
                }

            base.Update(gameTime);
            }

        #region Draw


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        protected override void Draw(GameTime gameTime)
            {
            graphics.GraphicsDevice.Clear(Color.Black);

            // The real drawing happens inside the screen manager component.
            base.Draw(gameTime);
            }


        #endregion
        }


    #region Entry Point

    /// <summary>
    /// The main entry point for the application.
    /// </summary>
    static class Program
        {
        static void Main()
            {
            using (GameStateManagementGame game = new GameStateManagementGame())
                {
                game.Run();
                }
            }
        }

    #endregion
}
