﻿#region File Description

#endregion

#region Using Statements
using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml.Serialization;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Content;
#endregion

namespace TetrisDreams
{
    public enum EGameState
        {
        eGameNotStarted = 0,
        eWaitingForBlocks = 1,
        eAddNewBlock = 2,
        eBlockInPlay = 3,
        eRemovingLines = 4,
        eLevelIncrease = 5,
        eGamePaused = 6,
        eCheckingScore = 7,
        eSavingScore = 8,
        eGameOver = 9
        };

    public enum SavingState
        {
        NotSaving,
        ReadyToSelectStorageDevice,
        SelectingStorageDevice,
        ReadyToOpenStorageContainer,    // once we have a storage device start here
        OpeningStorageContainer,
        ReadyToSave
        }

    public enum LoadingState
        {
        NotLoading,
        ReadyToSelectStorageDevice,
        SelectingStorageDevice,
        ReadyToOpenStorageContainer,    // once we have a storage device start here
        OpeningStorageContainer,
        ReadyToLoad
        }


    enum EHitTestResult
        {
        eHitNone = 0,
        eHitBlock = 1,
        eHitEdge = 2
        }
    

    public sealed class TetrisModel
    {
        #region Fields
        public static readonly TimeSpan[] levelSpeed =   { 
                                                    new TimeSpan(0,0,0,0,500),
                                                    new TimeSpan(0,0,0,0,450),
                                                    new TimeSpan(0,0,0,0,400),
                                                    new TimeSpan(0,0,0,0,350),
                                                    new TimeSpan(0,0,0,0,300),
                                                    new TimeSpan(0,0,0,0,250),
                                                    new TimeSpan(0,0,0,0,200),
                                                    new TimeSpan(0,0,0,0,150),
                                                    new TimeSpan(0,0,0,0,100),
                                                    new TimeSpan(0,0,0,0,075),
                                                    };

        public static readonly int[,] levelPointsPerBlock =  {
                                                         {6, 24},
                                                         {9, 27},
                                                         {12, 30},
                                                         {15, 33},
                                                         {18, 36},
                                                         {21, 39},
                                                         {24, 42},
                                                         {27, 45},
                                                         {30, 48},
                                                         {33, 51},
                                                         };


        private static readonly TetrisModel instance = new TetrisModel();

        SavingState saveState = SavingState.NotSaving;
        LoadingState loadState = LoadingState.NotLoading;


        StorageDevice storageDevice;
        IAsyncResult asyncResult;
        StorageContainer storageContainer;
        string filename = "savedata.xml";

        public TetrisBlock aBlock = new TetrisBlock(0, 0); 
             
        // We store all highscores, options etc in here
        public TetrisSaveData tetrisSaveData = new TetrisSaveData();
        
        public EBlockColor[,] gameBoardMatrix;
        public TetrisBlockQueue tetrisBlockQueue;
        public EGameState gameState { get; set; }
        public int blockShadowPos { get; private set; }
        public int tetrisLevel { get; private set; }
        public int tetrisScore { get; private set; }
        public int newHighScore { get; private set; }
        int tetrisCompletedLines;
        TimeSpan waitForBlockTime;
        TimeSpan timeForBlockWaited;
        bool resetTimeForBlockWaited;
        #endregion

        public static TetrisModel Instance
            {
            get
                {
                return instance;
                }
            }

        private TetrisModel()
            {                 
            newHighScore = -1;
            }

        public void NewGame()
            {
            // Reset everything
            tetrisLevel = 1;
            tetrisScore = 0;
            tetrisCompletedLines = 0;
            newHighScore = -1;

            resetTimeForBlockWaited = false;

            gameBoardMatrix = new EBlockColor[20, 10];
            tetrisBlockQueue = new TetrisBlockQueue();
            TetrisBlock aBlock = new TetrisBlock(0, 0);

            waitForBlockTime = TetrisModel.levelSpeed.ElementAt(0);
            gameState = EGameState.eGameNotStarted;
            }

        public bool AddBlock()
            {
            //
            if (gameState != EGameState.eAddNewBlock)
                {
                return false;
                }

            gameState = EGameState.eBlockInPlay;
            return true;
            }

        public bool FallBlock()
            {
            tetrisBlockQueue.NextBlock(ref aBlock);

            // Is the gameboard index at the pos one step down occupied?
            EHitTestResult hitResult = WillBlockHitGameBoardBlock(0, 1);
            if (hitResult == EHitTestResult.eHitBlock || hitResult == EHitTestResult.eHitEdge)
                {
                if (AddBlockToBoard())
                    {
                    tetrisBlockQueue.RemoveBlock();
                    gameState = EGameState.eWaitingForBlocks;
                    }
                else
                    {
                    gameState = EGameState.eCheckingScore;
                    }
                }

            aBlock.posY++;

            return true;
            }

        // TODO: Fix this UGLY method!!
        public bool RotateBlock(bool rotateRight)
            {
            if (gameState == EGameState.eBlockInPlay)
                {
                tetrisBlockQueue.NextBlock(ref aBlock);
                aBlock.Rotate(rotateRight);

                EHitTestResult hitResult = WillBlockHitGameBoardBlock(0, 0);
                if (hitResult == EHitTestResult.eHitEdge)
                    {
                    if (aBlock.posX > 5)
                        {
                        for (int i = -1; i > -3; i--)
                            {
                            hitResult = WillBlockHitGameBoardBlock(i, 0);
                                {
                                if (hitResult == EHitTestResult.eHitNone)
                                    {
                                    aBlock.posX += i;
                                    UpdateBlockShadowPos();
                                    return true;
                                    }
                                }
                            }
                        }
                    else
                        {
                        if (aBlock.posX < 3)
                            {
                            for (int i = 1; i < 3; i++)
                                {
                                hitResult = WillBlockHitGameBoardBlock(i, 0);
                                    {
                                    if (hitResult == EHitTestResult.eHitNone)
                                        {
                                        aBlock.posX += i;
                                        UpdateBlockShadowPos();
                                        return true;
                                        }

                                    }
                                }
                            }
                        }


                    // The rotatation was not valid. Reset to last position. I know it's ugly code :P
                    aBlock.Rotate(!rotateRight);
                    }
                else
                    if (hitResult == EHitTestResult.eHitBlock)
                        {
                        aBlock.Rotate(!rotateRight);
                        }
                }

            UpdateBlockShadowPos();
            return true;
            }

        public bool MoveBlock(int offsetX)
            {
            EHitTestResult hitResult = WillBlockHitGameBoardBlock(offsetX, 0);
            if (hitResult == EHitTestResult.eHitBlock || hitResult == EHitTestResult.eHitEdge)
                {
                return false;
                }

            tetrisBlockQueue.NextBlock(ref aBlock);
            aBlock.posX += offsetX;
            return true;
            }

        private bool BlockToBoardHitTest()
            {
            return true;
            }

        private bool AddBlockToBoard()
            {
            tetrisBlockQueue.NextBlock(ref aBlock);
            for (int i = 0; i < 4; i++)
                {
                for (int n = 0; n < 4; n++)
                    {
                    if ((int)aBlock.blockArray.GetValue(i, n) != 0)
                        {
                        if (aBlock.posY < 0)
                            {
                            return false;
                            }

                        gameBoardMatrix[i + aBlock.posY, n + aBlock.posX] = aBlock.blockColor;
                        }
                    }
                }

            // Add score for the block left on the gameboard
            tetrisScore += (int)TetrisModel.levelPointsPerBlock.GetValue(tetrisLevel - 1, 0);

            // Add score for any line removed
            int newTetrisLevel = 1;
            int removedLines = CheckLines();
            tetrisCompletedLines += removedLines;
            tetrisScore += (int)TetrisModel.levelPointsPerBlock.GetValue(tetrisLevel - 1, 1) * removedLines;

            if (tetrisCompletedLines < 10)
                newTetrisLevel = 1;
            else
                if (tetrisCompletedLines > 91)
                    newTetrisLevel = 10;
                else
                    newTetrisLevel = 1 + tetrisCompletedLines / 10;

            if (newTetrisLevel != tetrisLevel)
                {
                tetrisLevel = newTetrisLevel;
                waitForBlockTime = TetrisModel.levelSpeed.ElementAt(tetrisLevel - 1);
                }

            return true;
            }

        private EHitTestResult WillBlockHitGameBoardBlock(int offsetX, int offsetY)
            {
            for (int i = 0; i < 4; i++)
                {
                for (int n = 0; n < 4; n++)
                    {
                    tetrisBlockQueue.NextBlock(ref aBlock);
                    if ((int)aBlock.blockArray.GetValue(i, n) != 0)
                        {

                        int x, y;

                        x = aBlock.posX + n + offsetX;
                        y = aBlock.posY + i + offsetY;


                        //Check left, right and lower limits
                        if (x < 0 || x > 9 || y > 19)
                            return EHitTestResult.eHitEdge;

                        // New blocks just above the upper edge is OK
                        if (y < 0)
                            continue;

                        // Check if position is free from block
                        if (gameBoardMatrix[y, x] != EBlockColor.eNone)
                            return EHitTestResult.eHitBlock;
                        }
                    }
                }
            return EHitTestResult.eHitNone;
            }

        public int DropBlock()
            {
            int linesToFall;
            for (linesToFall = 1; linesToFall < 20; linesToFall++)
                {
                if (WillBlockHitGameBoardBlock(0, linesToFall) != 0)
                    {
                    break;
                    }

                }

            tetrisBlockQueue.NextBlock(ref aBlock);
            aBlock.posY += linesToFall - 1;

            //Give the player time to adjust the block that fell.
            resetTimeForBlockWaited = true;

            return -1;
            }

        // This method is a fix to make sure we can drop several blocks in a row without waiting for adjustment
        // Without this method the game will actually freze when holding down the drop key. 
        public int DropBlockAgain()
            {
            DropBlock();
            resetTimeForBlockWaited = false;

            return -1;
            }

        public int CheckLines()
            {
            int removedLines = 0;
            //ArrayList lineList = new ArrayList();


            for (int i = 0; i < 20; i++)
                {
                bool removeLine = true;
                for (int n = 0; n < 10; n++)
                    {
                    if (gameBoardMatrix[i, n] == EBlockColor.eNone)
                        {
                        removeLine = false;
                        }
                    }
                if (removeLine)
                    {
                    RemoveLines(i);
                    removedLines++;
                    }
                }

            if (removedLines > 0)
            // Play sfx
                {
                if (TetrisModel.Instance.tetrisSaveData.sfx)
                    GameBoardScreen.PlaySfx();
                }

            return removedLines;
            }

        public bool RemoveLines(int index)
            {
            for (int i = index; i > 0; i--)
                {
                for (int n = 0; n < 10; n++)
                    {
                    gameBoardMatrix[i, n] = gameBoardMatrix[i - 1, n];
                    }
                }

            return true;
            }

        public void UpdateBlockShadowPos()
            {
            for (blockShadowPos = 1; blockShadowPos < 20; blockShadowPos++)
                {
                if (WillBlockHitGameBoardBlock(0, blockShadowPos) != 0)
                    {
                    break;
                    }
                }
            }


        public bool CheckScoreToHighScore(String name, int score)
            {
            bool result = false;
            HighscorePost post = new HighscorePost(name, score);

            if (tetrisSaveData.highscore.Count >= 10)
                {
                if (tetrisSaveData.highscore.ElementAt(9).score < score)
                    {
                    tetrisSaveData.highscore.RemoveAt(9);
                    tetrisSaveData.highscore.Add(post);
                    tetrisSaveData.highscore.Sort(HighScoreComparer);
                    result = true;
                    }
                }
            else
                {
                tetrisSaveData.highscore.Add(post);
                tetrisSaveData.highscore.Sort(HighScoreComparer);
                result = true;
                }
            
            if(result)
                {
                // Lets notify about a new highscore
                newHighScore = tetrisSaveData.highscore.IndexOf(post);
                
                // Lets start a save of the changes
                //DoGameSave();
                }
                
            return result;
            }

        public bool DoUpdate(GameTime gameTime)
            {
            bool result = true;

            switch (gameState)
                {
                case EGameState.eGameNotStarted:
                    {
                    gameState = EGameState.eWaitingForBlocks;
                    } break;
                case EGameState.eWaitingForBlocks:
                    {
                    if (gameTime.TotalGameTime - timeForBlockWaited > waitForBlockTime)
                        {
                        timeForBlockWaited = gameTime.TotalGameTime;
                        gameState = EGameState.eAddNewBlock;
                        }
                    } break;
                case EGameState.eAddNewBlock:
                    {
                    TetrisModel.Instance.AddBlock();
                    TetrisModel.Instance.UpdateBlockShadowPos();
                    } break;
                case EGameState.eBlockInPlay:
                    {
                    // We just dropped the block, give the player some extra time to adjust the horz pos
                    if (resetTimeForBlockWaited)
                        {
                        timeForBlockWaited = gameTime.TotalGameTime;
                        resetTimeForBlockWaited = false;
                        }

                    // Now check if time has passed to let the block fall one more step
                    if (gameTime.TotalGameTime - timeForBlockWaited >waitForBlockTime)
                        {
                        timeForBlockWaited = gameTime.TotalGameTime;
                        TetrisModel.Instance.FallBlock();
                        }

                    TetrisModel.Instance.UpdateBlockShadowPos();

                    } break;
                case EGameState.eRemovingLines:
                    {
                    TetrisModel.Instance.UpdateBlockShadowPos();
                    } break;
                case EGameState.eLevelIncrease:
                    {
                    } break;
                case EGameState.eGamePaused:
                    {
                    } break;
                case EGameState.eCheckingScore:
                    {
                    String name = "Guest";
                    if (Gamer.SignedInGamers[PlayerIndex.One] != null)
                        {
                        // Get the users avatar description
                        name = Gamer.SignedInGamers[PlayerIndex.One].Gamertag;
                        }
                    if (TetrisModel.Instance.CheckScoreToHighScore(name, tetrisScore))
                        gameState = EGameState.eSavingScore;
                    else
                        gameState = EGameState.eGameOver;
                    } break;
                case EGameState.eSavingScore:
                    {
                    if (DoGameSave())
                        gameState = EGameState.eGameOver;
                    } break;
                case EGameState.eGameOver:
                    {
                    result = false;           
                    } break;
                default:
                    {
                    } break;
                }   
                return result;
            }



        public bool DoGameSave()
            {
            bool result = false;
                
            switch (saveState)
                {
                case SavingState.NotSaving:
                case SavingState.ReadyToSelectStorageDevice:
                #if XBOX
                    if (!Guide.IsVisible)
                #endif
                        {
                        asyncResult = StorageDevice.BeginShowSelector(PlayerIndex.One, null, null);
                        saveState = SavingState.SelectingStorageDevice;
                        }
                        break;

                case SavingState.SelectingStorageDevice:
                    if (asyncResult.IsCompleted)
                        {
                        storageDevice = StorageDevice.EndShowSelector(asyncResult);
                        saveState = SavingState.ReadyToOpenStorageContainer;
                        }
                        break;

                case SavingState.ReadyToOpenStorageContainer:
                    if (storageDevice == null || !storageDevice.IsConnected)
                        {
                        saveState = SavingState.ReadyToSelectStorageDevice;
                        }
                    else
                        {
                        asyncResult = storageDevice.BeginOpenContainer("TetrisDreams", null, null);
                        saveState = SavingState.OpeningStorageContainer;
                        }
                        break;

                case SavingState.OpeningStorageContainer:
                    if (asyncResult.IsCompleted)
                        {
                        storageContainer = storageDevice.EndOpenContainer(asyncResult);
                        saveState = SavingState.ReadyToSave;
                        }
                        break;

                case SavingState.ReadyToSave:
                    if (storageContainer == null)
                        {
                        saveState = SavingState.ReadyToOpenStorageContainer;
                        }
                    else
                        {
                        try
                            {
                            DeleteExisting();
                            Save();
                            result = true;
                            }
                        catch (IOException e)
                            {
                            // Replace with in game dialog notifying user of error
                            //Trace.WriteLine(e.Message);
                            Debug.WriteLine(e.Message);
                            result = false;
                            }
                        finally
                            {
                            storageContainer.Dispose();
                            storageContainer = null;
                            saveState = SavingState.NotSaving;
                            }
                        }
                        break;
            
                }

            return result;
            }


        private void DeleteExisting()
                {
                if (storageContainer.FileExists(filename))
                    {
                    storageContainer.DeleteFile(filename);
                    }
                }

        private void Save()
            {
            using (Stream stream = storageContainer.CreateFile(filename))
                {
                XmlSerializer serializer = new XmlSerializer(typeof(TetrisSaveData));
                serializer.Serialize(stream, tetrisSaveData);
                }
            }


        public bool DoGameLoad()
            {
            bool done = false;

            switch (loadState)
                {
                case LoadingState.NotLoading:
                case LoadingState.ReadyToSelectStorageDevice:
                #if XBOX
                    if (!Guide.IsVisible)
                #endif
                        {
                        asyncResult = StorageDevice.BeginShowSelector(PlayerIndex.One, null, null);
                        loadState = LoadingState.SelectingStorageDevice;
                        }
                        break;

                case LoadingState.SelectingStorageDevice:
                    if (asyncResult.IsCompleted)
                        {
                        storageDevice = StorageDevice.EndShowSelector(asyncResult);
                        loadState = LoadingState.ReadyToOpenStorageContainer;
                        }
                        break;

                case LoadingState.ReadyToOpenStorageContainer:
                    if (storageDevice == null || !storageDevice.IsConnected)
                        {
                            loadState = LoadingState.ReadyToSelectStorageDevice;
                        }
                    else
                        {
                        asyncResult = storageDevice.BeginOpenContainer("TetrisDreams", null, null);
                        loadState = LoadingState.OpeningStorageContainer;
                        }
                        break;

                case LoadingState.OpeningStorageContainer:
                    if (asyncResult.IsCompleted)
                        {
                        storageContainer = storageDevice.EndOpenContainer(asyncResult);
                        loadState = LoadingState.ReadyToLoad;
                        }
                        break;

                case LoadingState.ReadyToLoad:
                    if (storageContainer == null)
                        {
                            loadState = LoadingState.ReadyToOpenStorageContainer;
                        }
                    else
                        {
                        if (!storageContainer.FileExists(filename))
                            {
                            // If not, dispose of the container and return.
                            storageContainer.Dispose();
                            done = true;
                            loadState = LoadingState.NotLoading;
                            break;
                            }
                        try
                            {
                            Load();
                            done = true;
                            loadState = LoadingState.NotLoading;
                            }
                        catch (IOException e)
                            {
                            // Replace with in game dialog notifying user of error
                            //Trace.WriteLine(e.Message);
                            Debug.WriteLine(e.Message);
                            }
                        finally
                            {
                            storageContainer.Dispose();
                            storageContainer = null;
                            loadState = LoadingState.NotLoading;
                            }
                        }
                            break;
                }

                return done;
            }


        private void Load()
            {
            using (Stream stream = storageContainer.OpenFile(filename, FileMode.Open))
                {
                XmlSerializer serializer = new XmlSerializer(typeof(TetrisSaveData));
                tetrisSaveData = (TetrisSaveData)serializer.Deserialize(stream);
                }
            }


        private static int HighScoreComparer(HighscorePost postX, HighscorePost postY)
            {
            if(postX.score > postY.score)
                return -1;
            
            if(postX.score < postY.score)
                return 1;   
            
            //Todo fix sort by name if score is equal
            /*if(postX.score == postY.score)
                {
                if(postX.name > postY.name)
                    return 1;
                else
                    return -1;
                }*/        
                
            return 0;
            }

        public void StartMusic()
            {
            if(tetrisSaveData.music)
                {
                MediaPlayer.Volume = (float)tetrisSaveData.musicvolyme / 100;
                MediaPlayer.IsRepeating = true;
                MediaPlayer.Play(GameStateManagementGame.music);
                }
            }

        } // TetrisModel
} // TetrisDreams 
